@@ -35,6 +35,7 @@
 // Local Includes
 #include "application.h"
 #include "paneltreeview.h"
+#include "historymanager.h"
 #include "historymodels.h"
 #include "urlfilterproxymodel.h"
 
@@ -45,7 +46,7 @@
 #include <KMessageBox>
 
 // Qt Includes
-#include <QtGui/QHeaderView>
+#include <QHeaderView>
 
 
 HistoryPanel::HistoryPanel(const QString &title, QWidget *parent, Qt::WindowFlags flags)
