@@ -52,6 +52,11 @@
 #include <KMessageBox>
 #include <KStandardDirs>
 
+#include <config-kactivities.h>
+#ifdef HAVE_KACTIVITIES
+#include <KActivities/ResourceInstance>
+#endif
+
 // Qt Includes
 #include <QtGui/QLabel>
 #include <QtGui/QMovie>
@@ -160,40 +165,40 @@ QList<TabHistory> MainView::recentlyClosedTabs()
 
 void MainView::updateTabBarVisibility()
 {
-    if (ReKonfig::alwaysShowTabBar() || count() > 1)
+    // Get sure tabbar is well shown (and hided) during fullscreen navigation
+    // NOTE: don't ask me why, but it seems that using code like:
+    // MainWindow *w = qobject_cast<MainWindow *>(parent());
+    // does NOT work here. So, I'm asking you: WHY???
+    MainWindow *w = rApp->mainWindow();
+    if (w && !w->isFullScreen())
     {
-        // Get sure tabbar is well shown (and hided) during fullscreen navigation
-        // NOTE: don't ask me why, but it seems that using code like:
-        // MainWindow *w = qobject_cast<MainWindow *>(parent());
-        // does NOT work here. So, I'm asking you: WHY???
-        MainWindow *w = rApp->mainWindow();
-        if (w && !w->isFullScreen())
+        if (tabBar()->isHidden())
         {
-            if (tabBar()->isHidden())
-            {
-                tabBar()->show();
-            }
+            tabBar()->show();
+        }
 
-            // this to ensure tab button visibility also on new window creation
-            if (m_addTabButton->isHidden())
-            {
-                m_addTabButton->show();
-            }
+        // this to ensure tab button visibility also on new window creation
+        if (m_addTabButton->isHidden())
+        {
+            m_addTabButton->show();
         }
-    }
-    else
-    {
-        tabBar()->hide();
-        m_addTabButton->hide();
-        return;
     }
 }
 
 
 void MainView::updateAddTabButton()
 {
-    // update tab button position
+    if (!ReKonfig::alwaysShowTabBar())
+    {
+        bool b = (count() == 1);
 
+        tabBar()->setVisible(!b);
+        m_addTabButton->setVisible(!b);
+        if (b)
+            return;
+    }
+
+    // update tab button position
     int tabWidgetWidth = frameSize().width();
     int tabBarWidth = tabBar()->sizeHint().width();
 
@@ -206,10 +211,6 @@ void MainView::updateAddTabButton()
         setCornerWidget(0);
         m_addTabButton->move(tabBarWidth, 0);
     }
-
-    // Make sure the add tab button is correctly shown
-    // For some reason, it's being hidden during a "fixed pos to corner" change
-    m_addTabButton->show();
 }
 
 
@@ -259,6 +260,23 @@ void MainView::currentChanged(int index)
     // retrieve the old webview (that where we move from)
     WebTab *oldTab = this->webTab(m_currentTabIndex);
 
+    WebView *view = tab->view();
+
+#ifdef HAVE_KACTIVITIES
+    if (!QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled)) {
+        if (oldTab) {
+            WebView *oldView = oldTab->view();
+            if (activityResourceInstance(oldView)) {
+                activityResourceInstance(oldView)->notifyFocusedOut();
+            }
+        }
+
+        if (activityResourceInstance(view)) {
+            activityResourceInstance(view)->notifyFocusedIn();
+        }
+    }
+#endif
+
     // set current index
     m_currentTabIndex = index;
 
@@ -276,7 +294,7 @@ void MainView::currentChanged(int index)
     connect(tab->page(), SIGNAL(linkHovered(QString, QString, QString)),
             this, SIGNAL(linkHovered(QString)));
 
-    emit currentTitle(tab->view()->title());
+    emit currentTitle(view->title());
     m_widgetBar->setCurrentIndex(index);
 
     // clean up "status bar"
@@ -289,7 +307,7 @@ void MainView::currentChanged(int index)
     if (tab->url().scheme() == QL1S("about"))
         m_widgetBar->currentWidget()->setFocus();
     else
-        tab->view()->setFocus();
+        view->setFocus();
 
     tabBar()->resetTabHighlighted(index);
 }
@@ -645,6 +663,21 @@ void MainView::webViewUrlChanged(const QUrl &url)
     if (!tab)
         return;
 
+#ifdef HAVE_KACTIVITIES
+    if (
+            !QWebSettings::globalSettings()->testAttribute(QWebSettings::PrivateBrowsingEnabled) &&
+            !url.isEmpty() &&
+            url.scheme() != "about"
+        )
+    {
+        if (!activityResourceInstance(view)) {
+            new KActivities::ResourceInstance(window()->winId(), view);
+        }
+
+        activityResourceInstance(view)->setUri(url);
+    }
+#endif
+
     int index = indexOf(tab);
     if (ReKonfig::hoveringTabOption() == 2)
         tabBar()->setTabToolTip(index, url.toString());
@@ -805,3 +838,11 @@ void MainView::detachTab(int index, MainWindow *toWindow)
         connect(tab->page(), SIGNAL(printRequested(QWebFrame*)), w->mainView(), SIGNAL(printRequested(QWebFrame*)));
     }
 }
+
+#ifdef HAVE_KACTIVITIES
+KActivities::ResourceInstance * MainView::activityResourceInstance(WebView * view)
+{
+    return view->findChild<KActivities::ResourceInstance*> ();
+}
+#endif
+
