@@ -37,10 +37,13 @@
 #include "bookmarkmanager.h"
 #include "downloadmanager.h"
 #include "iconmanager.h"
+#include "historymanager.h"
 #include "historymodels.h"
 #include "mainview.h"
 #include "mainwindow.h"
 #include "previewselectorbar.h"
+#include "thumbupdater.h"
+#include "urlfilterproxymodel.h"
 #include "websnap.h"
 #include "webpage.h"
 #include "webtab.h"
@@ -50,20 +53,24 @@
 #include <KIconLoader>
 #include <KLocale>
 #include <KMimeType>
+#include <KRun>
 #include <KStandardDirs>
 
 // Qt Includes
 #include <QFile>
 #include <QAction>
 #include <QWebFrame>
+#include <QProcess>
 
 
 NewTabPage::NewTabPage(QWebFrame *frame)
     : QObject(frame)
     , m_root(frame->documentElement())
+    , m_showFullHistory(false)
 {
     QString htmlFilePath = KStandardDirs::locate("data", "rekonq/htmls/home.html");
-    QString imagesPath = QL1S("file://") + KGlobal::dirs()->findResourceDir("data", "rekonq/pics/bg.png") + QL1S("rekonq/pics");
+    QString dataPath = QL1S("file://") + htmlFilePath;
+    dataPath.remove(QL1S("/htmls/home.html"));
 
     QFile file(htmlFilePath);
     bool isOpened = file.open(QIODevice::ReadOnly);
@@ -74,13 +81,47 @@ NewTabPage::NewTabPage(QWebFrame *frame)
     else
     {
         m_html = file.readAll();
-        m_html.replace(QL1S("%2"), imagesPath);
+        m_html.replace(QL1S("$DEFAULT_PATH"), dataPath);
     }
 }
 
 
 void NewTabPage::generate(const KUrl &url)
 {
+    // about:preview links
+    if (KUrl("about:settings").isParentOf(url))
+    {
+        if (url.fileName() == QL1S("network"))
+        {
+            QString program = QL1S("kcmshell4");
+            QStringList arguments;
+            arguments << QL1S("kcm_networkmanagement");
+            QProcess *proc = new QProcess(parent());
+            proc->start(program, arguments);
+            return;
+        }
+
+        if (url.fileName() == QL1S("proxy"))
+        {
+            QString program = QL1S("kcmshell4");
+            QStringList arguments;
+            arguments << QL1S("proxy");
+            QProcess *proc = new QProcess(parent());
+            proc->start(program, arguments);
+            return;
+        }
+
+        if (url.fileName() == QL1S("firewall"))
+        {
+            QString program = QL1S("kcmshell4");
+            QStringList arguments;
+            arguments << QL1S("kcm-ufw");
+            QProcess *proc = new QProcess(parent());
+            proc->start(program, arguments);
+            return;
+        }
+    }
+
     // about:preview links
     if (KUrl("about:preview").isParentOf(url))
     {
@@ -97,25 +138,32 @@ void NewTabPage::generate(const KUrl &url)
             ReKonfig::setPreviewNames(names);
             ReKonfig::setPreviewUrls(urls);
 
-            // Why doesn't it work well ?
-            // m_root.appendInside(emptyPreview(names.length() - 1));
-            // Replacing with this:
-            generate(KUrl("about:favorites"));
+            loadPageForUrl(KUrl("about:favorites"));
 
             rApp->mainWindow()->currentTab()->createPreviewSelectorBar(index);
             return;
         }
+
         if (url.directory() == QL1S("preview/remove"))
         {
-            removePreview(url.fileName().toInt());
+            int index = url.fileName().toInt();
+            removePreview(index);
             return;
         }
+
         if (url.directory() == QL1S("preview/modify"))
         {
             int index = url.fileName().toInt();
             rApp->mainWindow()->currentTab()->createPreviewSelectorBar(index);
             return;
         }
+
+        if (url.directory() == QL1S("preview/reload"))
+        {
+            int index = url.fileName().toInt();
+            reloadPreview(index);
+            return;
+        }
     }
 
     // about:tabs links
@@ -125,8 +173,6 @@ void NewTabPage::generate(const KUrl &url)
         {
             const int winIndex = url.queryItem(QL1S("win")).toInt();
             const int tabIndex = url.queryItem(QL1S("tab")).toInt();
-            kDebug() << "win = " << winIndex;
-            kDebug() << "tab = " << tabIndex;
 
             MainWindow *w = rApp->mainWindowList().at(winIndex).data();
 
@@ -144,12 +190,10 @@ void NewTabPage::generate(const KUrl &url)
         {
             const int winIndex = url.queryItem(QL1S("win")).toInt();
             const int tabIndex = url.queryItem(QL1S("tab")).toInt();
-            kDebug() << "win = " << winIndex;
-            kDebug() << "tab = " << tabIndex;
 
             MainWindow *w = rApp->mainWindowList().at(winIndex).data();
             w->mainView()->closeTab(tabIndex);
-            generate(KUrl("about:tabs"));
+            loadPageForUrl(KUrl("about:tabs"));
             return;
         }
     }
@@ -160,31 +204,92 @@ void NewTabPage::generate(const KUrl &url)
         if (url.fileName() == QL1S("restore"))
         {
             const int tabIndex = url.queryItem(QL1S("tab")).toInt();
-            kDebug() << "tab = " << tabIndex;
 
             rApp->mainWindow()->mainView()->restoreClosedTab(tabIndex, false);
             return;
         }
     }
 
-    if (url == KUrl("about:downloads/clear"))
+    // about:history links
+    if (KUrl("about:history").isParentOf(url))
     {
-        rApp->downloadManager()->clearDownloadsHistory();
-        generate(KUrl("about:downloads"));
-        return;
+        if (url.fileName() == QL1S("clear"))
+        {
+            rApp->historyManager()->clear();
+            loadPageForUrl(KUrl("about:history"));
+            return;
+        }
+
+        if (url.fileName() == QL1S("showAllItems"))
+        {
+            m_showFullHistory = true;
+            loadPageForUrl(KUrl("about:history"));
+            return;
+        }
+
+        if (url.fileName() == QL1S("search"))
+        {
+            QString value = url.queryItemValue(QL1S("q"));
+            loadPageForUrl(KUrl("about:history"), value);
+            return;
+        }
+
     }
-    if (url == KUrl("about:history/clear"))
+
+    // about:downloads links
+    if (KUrl("about:downloads").isParentOf(url))
     {
-        rApp->historyManager()->clear();
-        generate(KUrl("about:history"));
-        return;
+        if (url.fileName() == QL1S("clear"))
+        {
+            rApp->downloadManager()->clearDownloadsHistory();
+            loadPageForUrl(KUrl("about:downloads"));
+            return;
+        }
+
+        if (url.fileName() == QL1S("search"))
+        {
+            QString value = url.queryItemValue(QL1S("q"));
+            loadPageForUrl(KUrl("about:downloads"), value);
+            return;
+        }
+
+        if (url.fileName() == QL1S("opendir"))
+        {
+            QString value = url.queryItemValue(QL1S("q"));
+            KUrl dirUrl = KUrl(value);
+            (void)new KRun(dirUrl, rApp->mainWindow(), 0, dirUrl.isLocalFile());
+            return;
+        }
+
+        if (url.fileName() == QL1S("removeItem"))
+        {
+            int value = url.queryItemValue(QL1S("item")).toInt();
+            rApp->downloadManager()->removeDownloadItem(value);
+            loadPageForUrl(KUrl("about:downloads"));
+            return;
+        }
     }
+
     if (url == KUrl("about:bookmarks/edit"))
     {
         rApp->bookmarkManager()->slotEditBookmarks();
         return;
     }
 
+
+    if (url == KUrl("about:favorites/save"))
+    {
+        saveFavorites();
+        return;
+    }
+
+
+    loadPageForUrl(url);
+}
+
+
+void NewTabPage::loadPageForUrl(const KUrl &url, const QString & filter)
+{
     // webFrame can be null. See bug:282092
     QWebFrame *parentFrame = qobject_cast<QWebFrame *>(parent());
     if (!parentFrame)
@@ -206,6 +311,9 @@ void NewTabPage::generate(const KUrl &url)
         favoritesPage();
         updateWindowIcon();
         title = i18n("Favorites");
+        m_root.document().findFirst(QL1S("title")).setPlainText(title);
+        initJS();
+        return;
     }
     else if (encodedUrl == QByteArray("about:closedTabs"))
     {
@@ -215,7 +323,7 @@ void NewTabPage::generate(const KUrl &url)
     }
     else if (encodedUrl == QByteArray("about:history"))
     {
-        historyPage();
+        historyPage(filter);
         updateWindowIcon();
         title = i18n("History");
     }
@@ -227,7 +335,7 @@ void NewTabPage::generate(const KUrl &url)
     }
     else if (encodedUrl == QByteArray("about:downloads"))
     {
-        downloadsPage();
+        downloadsPage(filter);
         updateWindowIcon();
         title = i18n("Downloads");
     }
@@ -304,10 +412,11 @@ void NewTabPage::favoritesPage()
 {
     m_root.addClass(QL1S("favorites"));
 
-    const QWebElement add = createLinkItem(i18n("Add Favorite"),
-                                           QL1S("about:preview/add"),
-                                           QL1S("list-add"),
-                                           KIconLoader::Toolbar);
+    QWebElement add = createLinkItem(i18n("Add Favorite"),
+                                     QL1S("about:preview/add"),
+                                     QL1S("list-add"),
+                                     KIconLoader::Toolbar);
+    add.setAttribute(QL1S("class"), QL1S("right"));
     m_root.document().findFirst("#actions").appendInside(add);
 
     QStringList names = ReKonfig::previewNames();
@@ -326,29 +435,50 @@ void NewTabPage::favoritesPage()
 
         QWebElement prev = url.isEmpty()
                            ? emptyPreview(i)
-                           : validPreview(i, url, QString::number(i + 1) + " - " + names.at(i));
+                           : validPreview(i, url, QString::number(i + 1) + QL1S(" - ") + names.at(i));
 
         m_root.appendInside(prev);
     }
 }
 
 
-void NewTabPage::historyPage()
+void NewTabPage::historyPage(const QString & filter)
 {
     m_root.addClass(QL1S("history"));
 
-    const QWebElement clearHistory = createLinkItem(i18n("Clear History"),
-                                     QL1S("about:history/clear"),
-                                     QL1S("edit-clear"),
-                                     KIconLoader::Toolbar);
+    QWebElement searchForm = createFormItem(i18n("Search History"), QL1S("about:history/search"));
+    searchForm.setAttribute(QL1S("class"), QL1S("left"));
+
+    m_root.document().findFirst(QL1S("#actions")).appendInside(searchForm);
+
+    QWebElement clearHistory = createLinkItem(i18n("Clear History"),
+                               QL1S("about:history/clear"),
+                               QL1S("edit-clear"),
+                               KIconLoader::Toolbar);
+    clearHistory.setAttribute(QL1S("class"), QL1S("right"));
     m_root.document().findFirst(QL1S("#actions")).appendInside(clearHistory);
 
     HistoryTreeModel *model = rApp->historyManager()->historyTreeModel();
+    UrlFilterProxyModel *proxy = new UrlFilterProxyModel(this);
+    proxy->setSourceModel(model);
 
-    if (model->rowCount() == 0)
+    bool filterIsEmpty = filter.isEmpty();
+
+    if (!filterIsEmpty)
+        proxy->setFilterFixedString(filter);
+
+    if (proxy->rowCount() == 0)
     {
-        m_root.addClass(QL1S("empty"));
-        m_root.setPlainText(i18n("Your browsing history is empty"));
+        if (filterIsEmpty)
+        {
+            m_root.addClass(QL1S("empty"));
+            m_root.setPlainText(i18n("Your browsing history is empty"));
+        }
+        else
+        {
+            m_root.addClass(QL1S("empty"));
+            m_root.setPlainText(i18n("No matches for string %1 in history", filter));
+        }
         return;
     }
 
@@ -357,17 +487,17 @@ void NewTabPage::historyPage()
     const int truncateSize = 100;
     do
     {
-        QModelIndex index = model->index(i, 0, QModelIndex());
-        if (model->hasChildren(index))
+        QModelIndex index = proxy->index(i, 0, QModelIndex());
+        if (proxy->hasChildren(index))
         {
             m_root.appendInside(markup(QL1S("h3")));
             m_root.lastChild().setPlainText(index.data().toString());
 
-            m_root.appendInside(markup(QL1S(".folder")));
+            m_root.appendInside(markup(QL1S(".historyfolder")));
             QWebElement little = m_root.lastChild();
-            for (int j = 0; j < model->rowCount(index); ++j)
+            for (int j = 0; j < proxy->rowCount(index); ++j)
             {
-                QModelIndex son = model->index(j, 0, index);
+                QModelIndex son = proxy->index(j, 0, index);
                 KUrl u = son.data(HistoryModel::UrlStringRole).toUrl();
 
                 little.appendInside(son.data(HistoryModel::DateTimeRole).toDateTime().toString("hh:mm"));
@@ -392,8 +522,18 @@ void NewTabPage::historyPage()
             }
         }
         i++;
+        if (filterIsEmpty && m_showFullHistory == false && (i == 2))
+        {
+            m_root.appendInside(markup(QL1S("a")));
+            m_root.lastChild().setAttribute(QL1S("class") , QL1S("greybox"));
+            m_root.lastChild().setAttribute(QL1S("href") , QL1S("about:history/showAllItems"));
+            m_root.lastChild().setPlainText(i18n("Show full History"));
+            return;
+        }
     }
-    while (model->hasIndex(i , 0 , QModelIndex()));
+    while (proxy->hasIndex(i , 0 , QModelIndex()));
+
+    m_showFullHistory = false;
 }
 
 
@@ -401,10 +541,11 @@ void NewTabPage::bookmarksPage()
 {
     m_root.addClass(QL1S("bookmarks"));
 
-    const QWebElement editBookmarks = createLinkItem(i18n("Edit Bookmarks"),
-                                      QL1S("about:bookmarks/edit"),
-                                      QL1S("bookmarks-organize"),
-                                      KIconLoader::Toolbar);
+    QWebElement editBookmarks = createLinkItem(i18n("Edit Bookmarks"),
+                                QL1S("about:bookmarks/edit"),
+                                QL1S("bookmarks-organize"),
+                                KIconLoader::Toolbar);
+    editBookmarks.setAttribute(QL1S("class"), QL1S("right"));
     m_root.document().findFirst(QL1S("#actions")).appendInside(editBookmarks);
 
     KBookmarkGroup bookGroup = rApp->bookmarkManager()->rootGroup();
@@ -416,9 +557,15 @@ void NewTabPage::bookmarksPage()
     }
 
     KBookmark bookmark = bookGroup.first();
+
+    m_root.appendInside(markup(QL1S(".bookmarkfolder")));
+    QWebElement rootFolder = m_root.lastChild();
+    rootFolder.appendInside(markup(QL1S("h4")));
+    rootFolder.lastChild().setPlainText(i18n("Unsorted"));
+
     while (!bookmark.isNull())
     {
-        createBookItem(bookmark, m_root);
+        createBookmarkItem(bookmark, rootFolder);
         bookmark = bookGroup.next(bookmark);
     }
 }
@@ -430,7 +577,6 @@ void NewTabPage::closedTabsPage()
 
     QList<TabHistory> links = rApp->mainWindow()->mainView()->recentlyClosedTabs();
 
-    kDebug() << "CLOSED TABS: " << links.count();
     if (links.isEmpty())
     {
         m_root.addClass(QL1S("empty"));
@@ -441,7 +587,6 @@ void NewTabPage::closedTabsPage()
     for (int i = 0; i < links.count(); ++i)
     {
         TabHistory item = links.at(i);
-        kDebug() << "URL " << i << " : " << item.url;
         QWebElement prev;
 
         if (item.url.isEmpty())
@@ -450,24 +595,35 @@ void NewTabPage::closedTabsPage()
         prev = closedTabPreview(i, item.url, item.title);
 
         prev.setAttribute(QL1S("id"),  QL1S("preview") + QVariant(i).toString());
-        hideControls(prev);
+
+        // hide controls
+        prev.findFirst(QL1S(".right")).setStyleProperty(QL1S("visibility"), QL1S("hidden"));
+        prev.findFirst(QL1S(".left")).setStyleProperty(QL1S("visibility"), QL1S("hidden"));
+
         m_root.appendInside(prev);
     }
 }
 
 
-void NewTabPage::downloadsPage()
+void NewTabPage::downloadsPage(const QString & filter)
 {
     m_root.addClass(QL1S("downloads"));
 
-    const QWebElement clearDownloads = createLinkItem(i18n("Clear Downloads"),
-                                       QL1S("about:downloads/clear"),
-                                       QL1S("edit-clear"),
-                                       KIconLoader::Toolbar);
+    QWebElement searchForm = createFormItem(i18n("Search Downloads"), QL1S("about:downloads/search"));
+    searchForm.setAttribute(QL1S("class"), QL1S("left"));
+    m_root.document().findFirst(QL1S("#actions")).appendInside(searchForm);
+
+    QWebElement clearDownloads = createLinkItem(i18n("Clear Downloads"),
+                                 QL1S("about:downloads/clear"),
+                                 QL1S("edit-clear"),
+                                 KIconLoader::Toolbar);
+    clearDownloads.setAttribute(QL1S("class"), QL1S("right"));
     m_root.document().findFirst(QL1S("#actions")).appendInside(clearDownloads);
 
     DownloadList list = rApp->downloadManager()->downloads();
 
+    bool filterIsEmpty = filter.isEmpty();
+
     if (list.isEmpty())
     {
         m_root.addClass(QL1S("empty"));
@@ -475,15 +631,26 @@ void NewTabPage::downloadsPage()
         return;
     }
 
+    int i = 0;
+
     Q_FOREACH(DownloadItem * item, list)
     {
+        KUrl u = item->destUrl();
+        QString fName = u.fileName();
+
+        QString srcUrl = item->originUrl();
+
+        if (!filterIsEmpty)
+        {
+            if (!fName.contains(filter, Qt::CaseInsensitive) && !srcUrl.contains(filter, Qt::CaseInsensitive))
+                continue;
+        }
+
         m_root.prependInside(markup(QL1S("div")));
 
         QWebElement div = m_root.firstChild();
         div.addClass(QL1S("download"));
 
-        KUrl u = KUrl(item->destinationUrl());
-        QString fName = u.fileName();
         QString dir = u.directory();
         QString file = dir + QL1C('/') + fName;
 
@@ -499,24 +666,65 @@ void NewTabPage::downloadsPage()
         div.appendInside(QL1S("<em>") + date +  QL1S("</em>"));
         div.appendInside(QL1S("<br />"));
 
-        div.appendInside(QL1S("<a href=") + item->originUrl() +  QL1C('>') + item->originUrl() +  QL1S("</a>"));
+        div.appendInside(QL1S("<a href=") + srcUrl +  QL1C('>') + srcUrl +  QL1S("</a>"));
         div.appendInside(QL1S("<br />"));
 
-        if (QFile::exists(file))
+        switch (item->state())
         {
-            div.appendInside(markup(QL1S("a")));
-            div.lastChild().setAttribute(QL1S("href"), QL1S("file://") + dir);
-            div.lastChild().setPlainText(i18n("Open directory"));
+        case DownloadItem::KGetManaged:
+            div.appendInside(QL1S("<em>") + i18n("This download is managed by KGet. Check it to grab information about its state") +  QL1S("</em>"));
+            break;
+
+        case DownloadItem::Suspended:
+            div.appendInside(QL1S("<em>") + i18n("Suspended") +  QL1S("</em>"));
+            break;
+
+        case DownloadItem::Downloading:
+            div.appendInside(QL1S("<em>") + i18n("Downloading now...") +  QL1S("</em>"));
+            break;
+
+        case DownloadItem::Errors:
+            div.appendInside(QL1S("<em>") + i18nc("%1 = Error description", "Error: %1", item->errorString()) +  QL1S("</em>"));
+            break;
+
+        case DownloadItem::Done:
+        default:
+            if (QFile::exists(file))
+            {
+                div.appendInside(markup(QL1S("a")));
+                div.lastChild().setAttribute(QL1S("class"), QL1S("greylink"));
+                div.lastChild().setAttribute(QL1S("href"), QL1S("about:downloads/opendir?q=") + QL1S("file://") + dir);
+                div.lastChild().setPlainText(i18n("Open directory"));
+
+                div.appendInside(QL1S(" - "));
+
+                div.appendInside(markup(QL1S("a")));
+                div.lastChild().setAttribute(QL1S("class"), QL1S("greylink"));
+                div.lastChild().setAttribute(QL1S("href"), QL1S("file://") + file);
+                div.lastChild().setPlainText(i18n("Open file"));
+            }
+            else
+            {
+                div.appendInside(QL1S("<em>") + QL1S("Removed") +  QL1S("</em>"));
+            }
 
             div.appendInside(QL1S(" - "));
+
             div.appendInside(markup(QL1S("a")));
-            div.lastChild().setAttribute(QL1S("href"), QL1S("file://") + file);
-            div.lastChild().setPlainText(i18n("Open file"));
-        }
-        else
-        {
-            div.appendInside(QL1S("<em>") + QL1S("Removed") +  QL1S("</em>"));
+            div.lastChild().setAttribute(QL1S("class"), QL1S("greylink"));
+            div.lastChild().setAttribute(QL1S("href"), QL1S("about:downloads/removeItem?item=") + QString::number(i));
+            div.lastChild().setPlainText(i18n("Remove from list"));
+
+            break;
         }
+
+        i++;
+    }
+
+    if (i == 0)
+    {
+        m_root.addClass(QL1S("empty"));
+        m_root.setPlainText(i18n("No matches for string %1 in downloads", filter));
     }
 }
 
@@ -529,7 +737,7 @@ void NewTabPage::tabsPage()
     Q_FOREACH(const QWeakPointer<MainWindow> &wPointer, rApp->mainWindowList())
     {
         m_root.appendInside(markup(QL1S("h3")));
-        m_root.lastChild().setPlainText("Window");
+        m_root.lastChild().setPlainText(i18n("Window"));
 
         MainWindow *w = wPointer.data();
 
@@ -572,66 +780,25 @@ QWebElement NewTabPage::emptyPreview(int index)
     prev.findFirst(QL1S("a")).setAttribute(QL1S("href"),
                                            QL1S("about:preview/modify/") + QVariant(index).toString());
 
-    setupPreview(prev, index);
+    setupPreview(prev, index, false);
 
     return prev;
 }
 
 
-// NOTE: comment this out WITHOUT really deleting. May be of inspiration...
-// QWebElement NewTabPage::loadingPreview(int index, const KUrl &url)
-// {
-//     QWebElement prev = markup(QL1S(".thumbnail"));
-//
-//     prev.findFirst(QL1S(".preview img")).setAttribute(QL1S("src"),
-//             QL1S("file:///") + KStandardDirs::locate("appdata", "pics/busywidget.gif"));
-//     prev.findFirst(QL1S("span a")).setPlainText(i18n("Loading Preview..."));
-//     prev.findFirst(QL1S("a")).setAttribute(QL1S("href"), url.toMimeDataString());
-//
-//     setupPreview(prev, index);
-//     showControls(prev);
-//
-//     // NOTE: we need the page frame for two reasons
-//     // 1) to link to the WebPage calling the snapFinished slot
-//     // 2) to "auto-destroy" snaps on tab closing :)
-//     QWebFrame *frame = qobject_cast<QWebFrame *>(parent());
-//     WebSnap *snap = new WebSnap(url, frame);
-//     connect(snap, SIGNAL(snapDone(bool)), frame->page(), SLOT(updateImage(bool)), Qt::UniqueConnection);
-//     return prev;
-// }
-//
-//
-// void NewTabPage::updateThumbs()
-// {
-//     // Update page, but only if open
-//     if (m_root.document().findAll(QL1S("#rekonq-newtabpage")).count() == 0)
-//         return;
-//     if (m_root.findAll(QL1S(".favorites")).count() == 0 && m_root.findAll(QL1S(".closedTabs")).count() == 0)
-//         return;
-//
-//     QStringList urls = ReKonfig::previewUrls();
-//     QStringList names = ReKonfig::previewNames();
-//
-//     for (int i = 0; i < urls.count(); i++)
-//     {
-//         KUrl url = KUrl(urls.at(i));
-//         QString title = names.at(i);
-//
-//         if (WebSnap::existsImage(url))
-//         {
-//             QWebElement prev = m_root.findFirst(QL1S("#preview") + QVariant(i).toString());
-//             if (KUrl(prev.findFirst("a").attribute(QL1S("href"))) == url)
-//             {
-//                 QWebElement newPrev = validPreview(i, url, title);
-//
-//                 if (m_root.findAll(QL1S(".closedTabs")).count() != 0)
-//                     hideControls(newPrev);
-//
-//                 prev.replace(newPrev);
-//             }
-//         }
-//     }
-// }
+void NewTabPage::reloadPreview(int index)
+{
+    QString id = QL1S("#preview") + QString::number(index);
+    QWebElement thumb = m_root.document().findFirst(id);
+
+    QString urlString = ReKonfig::previewUrls().at(index);
+    QString nameString = ReKonfig::previewNames().at(index);
+
+    QString title = checkTitle(QString::number(index + 1) + QL1S(" - ") + nameString);
+
+    ThumbUpdater *t = new ThumbUpdater(thumb, urlString, title);
+    t->updateThumb();
+}
 
 
 QWebElement NewTabPage::validPreview(int index, const KUrl &url, const QString &title)
@@ -648,8 +815,7 @@ QWebElement NewTabPage::validPreview(int index, const KUrl &url, const QString &
     prev.findFirst(QL1S("span a")).setAttribute(QL1S("href"), url.toMimeDataString());
     prev.findFirst(QL1S("span a")).setPlainText(checkTitle(title));
 
-    setupPreview(prev, index);
-    showControls(prev);
+    setupPreview(prev, index, true);
     return prev;
 }
 
@@ -667,8 +833,10 @@ QWebElement NewTabPage::tabPreview(int winIndex, int tabIndex, const KUrl &url,
     prev.findFirst(QL1S("span a")).setPlainText(checkTitle(title));
 
     setupTabPreview(prev, winIndex, tabIndex);
-    prev.findFirst(QL1S(".remove")).setStyleProperty(QL1S("visibility"), QL1S("visible"));
-    prev.findFirst(QL1S(".modify")).setStyleProperty(QL1S("visibility"), QL1S("hidden"));
+
+    prev.findFirst(QL1S(".right")).setStyleProperty(QL1S("visibility"), QL1S("visible"));
+    prev.findFirst(QL1S(".left")).setStyleProperty(QL1S("visibility"), QL1S("hidden"));
+
     return prev;
 }
 
@@ -689,47 +857,44 @@ QWebElement NewTabPage::closedTabPreview(int index, const KUrl &url, const QStri
     prev.findFirst(QL1S("span a")).setAttribute(QL1S("href"), href);
     prev.findFirst(QL1S("span a")).setPlainText(checkTitle(title));
 
-    setupPreview(prev, index);
-    showControls(prev);
+    setupPreview(prev, index, true);
     return prev;
 }
 
 
-void NewTabPage::hideControls(QWebElement e)
+void NewTabPage::setupPreview(QWebElement e, int index, bool showControls)
 {
-    e.findFirst(QL1S(".remove")).setStyleProperty(QL1S("visibility"), QL1S("hidden"));
-    e.findFirst(QL1S(".modify")).setStyleProperty(QL1S("visibility"), QL1S("hidden"));
-}
+    e.findFirst(QL1S(".right img")).setAttribute(QL1S("src"),
+            QL1S("file:///") + KIconLoader::global()->iconPath("edit-delete", KIconLoader::DefaultState));
 
+    e.findFirst(QL1S(".right")).setAttribute(QL1S("title"), i18n("Remove favorite"));
 
-void NewTabPage::showControls(QWebElement e)
-{
-    e.findFirst(QL1S(".remove")).setStyleProperty(QL1S("visibility"), QL1S("visible"));
-    e.findFirst(QL1S(".modify")).setStyleProperty(QL1S("visibility"), QL1S("visible"));
-}
+    e.findFirst(QL1S(".left img")).setAttribute(QL1S("src"),
+            QL1S("file:///") + KIconLoader::global()->iconPath("view-refresh", KIconLoader::DefaultState));
 
+    e.findFirst(QL1S(".left")).setAttribute(QL1S("title"), i18n("Reload thumbnail"));
 
-void NewTabPage::setupPreview(QWebElement e, int index)
-{
-    e.findFirst(QL1S(".remove img")).setAttribute(QL1S("src"), QL1S("file:///") + KIconLoader::global()->iconPath("edit-delete", KIconLoader::DefaultState));
-    e.findFirst(QL1S(".remove")).setAttribute(QL1S("title"), QL1S("Remove favorite"));
-    e.findFirst(QL1S(".modify img")).setAttribute(QL1S("src"), QL1S("file:///") + KIconLoader::global()->iconPath("insert-image", KIconLoader::DefaultState));
-    e.findFirst(QL1S(".modify")).setAttribute(QL1S("title"), QL1S("Set new favorite"));
-
-    e.findFirst(QL1S(".modify")).setAttribute(QL1S("href"), QL1S("about:preview/modify/") + QVariant(index).toString());
-    e.findFirst(QL1S(".remove")).setAttribute(QL1S("href"), QL1S("about:preview/remove/") + QVariant(index).toString());
+    e.findFirst(QL1S(".left")).setAttribute(QL1S("href"), QL1S("about:preview/reload/") + QVariant(index).toString());
+    e.findFirst(QL1S(".right")).setAttribute(QL1S("href"), QL1S("about:preview/remove/") + QVariant(index).toString());
 
     e.setAttribute(QL1S("id"), QL1S("preview") + QVariant(index).toString());
+
+    if (showControls)
+    {
+        e.findFirst(QL1S(".right")).setStyleProperty(QL1S("visibility"), QL1S("visible"));
+        e.findFirst(QL1S(".left")).setStyleProperty(QL1S("visibility"), QL1S("visible"));
+    }
 }
 
 
 void NewTabPage::setupTabPreview(QWebElement e, int winIndex, int tabIndex)
 {
-    e.findFirst(QL1S(".remove img")).setAttribute(QL1S("src"), QL1S("file:///") + KIconLoader::global()->iconPath("edit-delete", KIconLoader::DefaultState));
-    e.findFirst(QL1S(".remove")).setAttribute(QL1S("title"), QL1S("Close Tab"));
+    e.findFirst(QL1S(".right img")).setAttribute(QL1S("src"),
+            QL1S("file:///") + KIconLoader::global()->iconPath("edit-delete", KIconLoader::DefaultState));
+    e.findFirst(QL1S(".right")).setAttribute(QL1S("title"), QL1S("Close Tab"));
 
     QString href = QL1S("about:tabs/remove?win=") + QString::number(winIndex) + QL1S("&tab=") + QString::number(tabIndex);
-    e.findFirst(QL1S(".remove")).setAttribute(QL1S("href"), href);
+    e.findFirst(QL1S(".right")).setAttribute(QL1S("href"), href);
 
     e.setAttribute(QL1S("id"), QL1S("win") + QString::number(winIndex) + QL1S("tab") + QString::number(tabIndex));
 }
@@ -746,31 +911,43 @@ void NewTabPage::removePreview(int index)
     ReKonfig::setPreviewNames(names);
     ReKonfig::setPreviewUrls(urls);
 
-    generate(KUrl("about:favorites"));
+    loadPageForUrl(KUrl("about:favorites"));
 
     ReKonfig::self()->writeConfig();
 }
 
 
-void NewTabPage::createBookItem(const KBookmark &bookmark, QWebElement parent)
+void NewTabPage::createBookmarkGroup(const KBookmark &bookmark, QWebElement parent)
+{
+    KBookmarkGroup group = bookmark.toGroup();
+    KBookmark bm = group.first();
+
+    parent.appendInside(markup(QL1S(".bookmarkfolder")));
+    QWebElement folder = parent.lastChild();
+    folder.appendInside(markup(QL1S("h4")));
+    folder.lastChild().setPlainText(group.fullText());
+
+    while (!bm.isNull())
+    {
+        createBookmarkItem(bm, folder);
+        bm = group.next(bm);
+    }
+}
+
+
+void NewTabPage::createBookmarkItem(const KBookmark &bookmark, QWebElement parent)
 {
     QString cacheDir = QL1S("file://") + KStandardDirs::locateLocal("cache" , "" , true);
     QString icon = QL1S("file://") + KGlobal::dirs()->findResource("icon", "oxygen/16x16/mimetypes/text-html.png");
+
     if (bookmark.isGroup())
     {
-        KBookmarkGroup group = bookmark.toGroup();
-        KBookmark bm = group.first();
-        parent.appendInside(markup(QL1S("h3")));
-        parent.lastChild().setPlainText(group.fullText());
-        parent.appendInside(markup(QL1S(".folder")));
-        while (!bm.isNull())
-        {
-            createBookItem(bm, parent.lastChild()); // it is .folder
-            bm = group.next(bm);
-        }
+        createBookmarkGroup(bookmark, m_root);
+        return;
     }
     else if (bookmark.isSeparator())
     {
+        kDebug() << "SEPARATOR";
         parent.appendInside(QL1S("<hr />"));
     }
     else
@@ -779,25 +956,27 @@ void NewTabPage::createBookItem(const KBookmark &bookmark, QWebElement parent)
         if (b.contains(QL1S("favicons")))
             icon = cacheDir + bookmark.icon() + QL1S(".png");
 
-        parent.appendInside(markup(QL1S("img")));
-        parent.lastChild().setAttribute(QL1S("src") , icon);
-        parent.lastChild().setAttribute(QL1S("width") , QL1S("16"));
-        parent.lastChild().setAttribute(QL1S("height") , QL1S("16"));
-        parent.appendInside(QL1S(" "));
         parent.appendInside(markup(QL1S("a")));
-        parent.lastChild().setAttribute(QL1S("href") , bookmark.url().prettyUrl());
-        parent.lastChild().setPlainText(bookmark.fullText());
-        parent.appendInside(QL1S("<br />"));
+        QWebElement bookmarkElement = parent.lastChild();
+        bookmarkElement.setAttribute(QL1S("href") , bookmark.url().prettyUrl());
+        bookmarkElement.addClass("bookmark");
+
+        bookmarkElement.appendInside(markup(QL1S("img")));
+        bookmarkElement.lastChild().setAttribute(QL1S("src") , icon);
+        bookmarkElement.lastChild().setAttribute(QL1S("width") , QL1S("16"));
+        bookmarkElement.lastChild().setAttribute(QL1S("height") , QL1S("16"));
+        bookmarkElement.appendInside(QL1S(" "));
+        bookmarkElement.appendInside(checkTitle(bookmark.fullText(), 40));
     }
 }
 
 
-QString NewTabPage::checkTitle(const QString &title)
+QString NewTabPage::checkTitle(const QString &title, int max)
 {
     QString t(title);
-    if (t.length() > 23)
+    if (t.length() > max)
     {
-        t.truncate(20);
+        t.truncate(max - 3);
         t +=  QL1S("...");
     }
     return t;
@@ -816,8 +995,100 @@ QWebElement NewTabPage::createLinkItem(const QString &title, const QString &urlS
 }
 
 
+QWebElement NewTabPage::createFormItem(const QString &title, const QString &urlString) const
+{
+    QWebElement form = markup(QL1S("form"));
+
+    form.setAttribute(QL1S("method"), QL1S("GET"));
+    form.setAttribute(QL1S("action"), urlString);
+
+    form.appendInside(markup(QL1S("input")));
+    form.lastChild().setAttribute(QL1S("type"), QL1S("text"));
+    form.lastChild().setAttribute(QL1S("name"), QL1S("q"));
+
+    form.appendInside(markup(QL1S("input")));
+    form.lastChild().setAttribute(QL1S("type"), QL1S("submit"));
+    form.lastChild().setAttribute(QL1S("value"), title);
+
+    return form;
+}
+
+
 void NewTabPage::updateWindowIcon()
 {
     int currentIndex = rApp->mainWindow()->mainView()->currentIndex();
     rApp->mainWindow()->changeWindowIcon(currentIndex);
 }
+
+
+void NewTabPage::initJS()
+{
+    QWebFrame *parentFrame = qobject_cast<QWebFrame *>(parent());
+    QString oldHTML = parentFrame->toHtml();
+
+    QString includes;
+    includes += QL1S("<head>");
+    includes += QL1S("<script src=\"$DEFAULT_PATH/htmls/jquery-1.7.2.min.js\" type=\"text/javascript\"></script>");
+    includes += QL1S("<script src=\"$DEFAULT_PATH/htmls/jquery-ui-1.8.20.custom.min.js\" type=\"text/javascript\"></script>");
+
+    QString htmlFilePath = KStandardDirs::locate("data", "rekonq/htmls/home.html");
+    QString dataPath = QL1S("file://") + htmlFilePath;
+    dataPath.remove(QL1S("/htmls/home.html"));
+
+    includes.replace(QL1S("$DEFAULT_PATH"), dataPath);
+
+    oldHTML.replace(QL1S("<head>"), includes);
+
+    QString javascript;
+    javascript += QL1S("<body>");
+    javascript += QL1S("<script>");
+    javascript += QL1S("$(function() {");
+    javascript += QL1S("    $( \"#content\" ).sortable({");
+    javascript += QL1S("        revert: true,");
+    javascript += QL1S("        cursor: \"move\",");
+    javascript += QL1S("        distance: 30,");
+    javascript += QL1S("        update: function(event, ui) { window.location.href = \"about:favorites/save\"; }");
+    javascript += QL1S("    });");
+    javascript += QL1S("    $( \".thumbnail\" ).disableSelection();");
+    javascript += QL1S("});");
+    javascript += QL1S("</script>");
+
+    oldHTML.replace(QL1S("<body>"), javascript);
+
+    parentFrame->setHtml(oldHTML);
+}
+
+
+void NewTabPage::saveFavorites()
+{
+    QStringList names = ReKonfig::previewNames();
+    QStringList urls = ReKonfig::previewUrls();
+
+    QStringList newNames = names;
+    QStringList newUrls = urls;
+
+    QWebElementCollection coll = m_root.document().findAll(QL1S(".thumbnail"));
+    QList<QWebElement> list = coll.toList();
+
+    int i = 0;
+
+    Q_FOREACH(QWebElement e, list)
+    {
+        if (!e.hasAttribute(QL1S("id")))
+            continue;
+
+        QString id = e.attribute(QL1S("id"));
+        kDebug() << "id: " << id;
+        int index = id.remove(QL1S("preview")).toInt();
+        kDebug() << "INDEX: " << index;
+
+        newNames.replace(i, names.at(index));
+        newUrls.replace(i, urls.at(index));
+        i++;
+    }
+
+    ReKonfig::setPreviewNames(newNames);
+    ReKonfig::setPreviewUrls(newUrls);
+
+    loadPageForUrl(KUrl("about:favorites"));
+}
